/**
 * @brief Este projeto demonstra a amostragem dos sinais de entrada pela tecnica polling
 * @author Wu Shin Ting
 * @date 30/09/2020
 *
 */

#include "derivative.h" /* include peripheral declarations */

#define GPIO_PIN(x)  ((1)<<(x)) ///< macro para gerar uma mascara com 1 no bit x

typedef enum estado_tag {
	DESABILITA,     /**< falso/apaga/desativa/liga */
	HABILITA,      /**< verdadeiro/acende/ativa/fecha */
} estado_type;

typedef struct rgb_bool_tag {
	estado_type vermelho;    
	estado_type verde;        
	estado_type azul;
} rgb_bool_type;

/**
 * @brief inicializa o componente ledRGB
 */
void GPIO_initLedRGB() {
	// Habilita os clocks dos mdulos
	SIM_SCGC5 |= SIM_SCGC5_PORTE_MASK ;    		// para leds

	// Configure pinos do led RGB com estado apagado
	// Funcao GPIO
	PORTE_PCR21 |= PORT_PCR_MUX(0x1);  // vermelho

	PORTE_PCR22 |= PORT_PCR_MUX(0x1);  // verde

	PORTE_PCR23 |= PORT_PCR_MUX(0x1);  // azul
	// Sentido do sinal: saida
	GPIOE_PDDR |= GPIO_PDDR_PDD(GPIO_PIN(21)) |
			GPIO_PDDR_PDD(GPIO_PIN(22)) |
			GPIO_PDDR_PDD(GPIO_PIN(23));
	// Limpar os bits	
	GPIOE_PCOR = GPIO_PCOR_PTCO(GPIO_PIN(21)) |
			GPIO_PCOR_PTCO(GPIO_PIN(22)) |
			GPIO_PCOR_PTCO(GPIO_PIN(23)) ;	
}

/**
 * @brief seta o estado do componente ledRGB
 * @param[in] estadoR HABILITA/DESABILITA led R
 * @param[in] estadoG HABILITA/DESABILITA led G
 * @param[in] estadoB HABILITA/DESABILITA led B
 */
void GPIO_ledRGB(rgb_bool_type cor) {
	if (cor.vermelho == HABILITA)
		GPIOE_PSOR = GPIO_PSOR_PTSO(GPIO_PIN(21));
	else 
		GPIOE_PCOR = GPIO_PCOR_PTCO(GPIO_PIN(21));

	if (cor.verde == HABILITA)
		GPIOE_PSOR = GPIO_PSOR_PTSO(GPIO_PIN(22));
	else
		GPIOE_PCOR = GPIO_PCOR_PTCO(GPIO_PIN(22));

	if (cor.azul == HABILITA)
		GPIOE_PSOR = GPIO_PSOR_PTSO(GPIO_PIN(23));
	else
		GPIOE_PCOR = GPIO_PCOR_PTCO(GPIO_PIN(23));     
}

/**
 * @brief inicializa o componente Botoeiras 
 */
void GPIO_initSwitches () {
	// Configure pinos das botoeiras
	SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK;				// para botoeiras   					// para leds

	//Funcao GPIO
	PORTA_PCR4 &= ~PORT_PCR_MUX(0x7);			// limpar os bits
	PORTA_PCR4 |= PORT_PCR_MUX(0x1);			// antes de setar 0b001

	PORTA_PCR5 |= PORT_PCR_MUX(0x1);

	PORTA_PCR12 |= PORT_PCR_MUX(0x1);

	//Sentido do sinal: entrada
	GPIOA_PDDR &= ~(GPIO_PDDR_PDD(GPIO_PIN(4)) |
			GPIO_PDDR_PDD(GPIO_PIN(5)) |
			GPIO_PDDR_PDD(GPIO_PIN(12)));    
}

/**
 * @brief amostre a chave NMI
 * @return HABILITA/DESABILITA
 */
estado_type GPIO_sampleSwNMI (){
	unsigned int a = GPIOA_PDIR;

	if (!(a & GPIO_PIN (4))) return (HABILITA);
	else return (DESABILITA);
}

/**
 * @brief amostre a chave IRQ5
 * @return HABILITA/DESABILITA
 */
estado_type GPIO_sampleSwIRQ5 (){
	unsigned int a = GPIOA_PDIR;

	if (!(a & GPIO_PIN (5))) return (HABILITA);
	else return (DESABILITA);
}

/**
 * @brief amostre a chave IRQ12
 * @return HABILITA/DESABILITA
 */
estado_type GPIO_sampleSwIRQ12 (){
	unsigned int a = GPIOA_PDIR;

	if (!(a & GPIO_PIN (12))) return (HABILITA);
	else return (DESABILITA);
}

/**
 * @brief controle o estado do led RGB pelas botoeiras.
 * @return 1 somente para satisfazer a sintaxe C
 */
int main (void)
{	
	rgb_bool_type cor[3]={{HABILITA,DESABILITA,DESABILITA},
			{DESABILITA,HABILITA,DESABILITA},
			{DESABILITA,DESABILITA,HABILITA}};
	
	enum cor_tag {
		VERMELHO,
		VERDE,
		AZUL
	};

	GPIO_initLedRGB();

	GPIO_initSwitches();

	for(;;) 
	{
		if (GPIO_sampleSwNMI() == HABILITA) {
			// liga led vermelho e apaga outros
			GPIO_ledRGB (cor[VERMELHO]);
		} 
		if (GPIO_sampleSwIRQ5() == HABILITA) {
			// liga led verde e apaga outros
			GPIO_ledRGB (cor[VERDE]);
		} 
		if (GPIO_sampleSwIRQ12() == HABILITA) {
			// liga led azul e apaga outros
			GPIO_ledRGB (cor[AZUL]);
		}
	}
	return 1;
}
